"""
Arc radon example code 
Copyright (C) 2016  Syed Tabish(syedtabish.a@gmail.com)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
"""

import cv2,time,sys,scipy
import numpy as np
from  scipy.misc import imsave
import matplotlib.pyplot as plt
import matplotlib.cm as cm
from _arc_radon import arc_radon as radon
from _arc_radon import pol2cart, inverse_radon, adjoint, arc_radon_ext, inverse_radon_ext


def fs_inverse_radon_ext(image, alpha, Rho, Phi, R):
    r = arc_radon_ext(image,alpha, Phi, Rho ,R)
    g = np.fft.fft(r, axis = 1)
    f_hat = inverse_radon_ext(g, R, alpha, int(Rho*0.9))
    Ir = np.fft.ifft(f_hat, axis=1).real
    cart_img = pol2cart(Ir)
    cart_img -= np.min(cart_img)
    cart_img *= 255.0/cart_img.max()
    name = 'reconstructed_image_ext.png'
    imsave(name,cart_img)
    return None


def fs_inverse_radon(image, alpha, Rho, Phi):
    R = max(image.shape)/2.0
    r = radon(image,alpha, Phi, Rho, R)
    g = np.fft.fft(r, axis = 1)
    f_hat = inverse_radon(g, R, alpha, int(Rho*0.9))
    Ir = np.fft.ifft(f_hat, axis=1).real
    cart_img = pol2cart(Ir)
    cart_img -= np.min(cart_img)
    cart_img *= 255.0/cart_img.max()
    name = 'reconstructed_image.png'
    imsave(name,cart_img)
    return None

def adjoint_inverse_radon(image, alpha, Rho, Phi):
    R = max(image.shape)/2.0
    r = radon(image,alpha, Phi, Rho, R)
    rmean = np.mean(r, axis=0)
    r -= rmean
    f_hat = adjoint(r, Rho)
    name = 'reconstructed_image_adjoint.png'
    imsave(name, f_hat)
    return None

if __name__ == '__main__':
    image = cv2.imread('phantom8.png',0)/255.
    alpha = 25
    Phi = 300 #Discretization in phi
    Rho = 300 #Discretization in rho
    #################################################################
    start_time = time.time();
    fs_inverse_radon(image, alpha, Rho, Phi)
    print 'Time Taken For Fourier inverse: (alpha = '+str(alpha)+') --> '+str(time.time() - start_time)+' s'
    #################################################################
    start_time = time.time();
    adjoint_inverse_radon(image, alpha, Rho, Phi)
    print 'Time Taken For Adjoint inverse: (alpha = '+str(alpha)+') --> '+str(time.time() - start_time)+' s'
    #################################################################
    image = cv2.imread('phantom21.png',0)/255.
    R = 75
    start_time = time.time();
    fs_inverse_radon_ext(image, alpha, Rho, Phi,R)
    print 'Time Taken For Fourier inverse: (alpha = '+str(alpha)+') --> '+str(time.time() - start_time)+' s'
